package iBus.example.hello;

import iBus.*;
import iBus.exception.*;

/**
 * Hello world example
 *
 * This is the most simple iBus listener application. All it does is
 * listening for postings sent to the hello_world channel, and display
 * them to System.err
 *
 * <P>Usage: hello_listener
 */
public class hello_listener {

    public static void main(String argv[])
      throws Exception
    {
	// a destination URL and a protocol stack for reliable multicast:
	final iBusURL url = iBusURLFactory.create("iBus.example.hello",
						  "1.0", "/messages");
	final Stack stack = new Stack("Reliable");

	// create a receiver object that dispatches incoming postings:
	HelloReceiver rcv = new HelloReceiver();

	// subscribe the receiver object to the hello world channel:
	stack.subscribe(url, rcv);

	System.err.println(Application.getAppName() + " is ready.");

	// wait for incoming postings:
	stack.waitTillExit();
    }
}

/**
 * This class is needed for receiving Postings on the hello_world channel.
 */
class HelloReceiver implements iBus.Receiver {
    /**
     * dispatchPush is invoked by iBus when a poting has arrived
     */
    public void dispatchPush(iBusURL source, Posting p)
    {
	String val =(String)p.getObject(0);
	System.err.println("got a message from " + source.getHostName()
			   + ": " + val);
	System.err.flush();
    }

    public Posting dispatchPull(iBusURL channel, Posting request)
    {
	return null;
    }

    public void error(iBusURL url, String details)
    {
	System.err.println("Error: " + details);
	System.err.flush();
	System.exit(1);
    }
}
