package iBus.example.membership;

import iBus.*;
import iBus.exception.*;
import java.io.Serializable;

/**
 * Membership listener example.
 *
 * The listener subscribes to a channel. It prints information to 
 * System.err whenever a listener process is started or terminated.
 *
 * Usage: listener
 */
public class listener {
    public static void main(String argv[])
    {
	Application.setAppName("listener");

	iBusURL url = null;
	final Stack stack = new Stack("Reliable");

	try {
	    url = iBusURLFactory.create("iBus.example.membership", 
					"1.0", "/members");
	} catch (Exception e) {
	    log_.panic("main: internal error: " + e);
	}
	
	// create an instance of the membership monitor:
	MembershipMon mon = new MembershipMon();

	// register monitor with stack and url:
	try {
	    stack.registerMonitor(url, mon);
	} catch (Exception e) {
	    log_.panic("main: registerMonitor failed: " + e);
	}

	// register as a talker. This leads to a view change:
	try {
	    stack.registerTalker(url);
	} catch (Exception e) {
	    log_.panic("main: registerTalker failed: " + e);
	}

	// block main thread:
	stack.waitTillExit();
    }
    
    // Log object:
    static Log log_ = new Log("listener");
}

/**
 * An implementation of iBus.Membership. Used for receiving
 * membership changes.
 */
class MembershipMon implements iBus.Membership {

    public Serializable getState(iBusURL channel) 
    {
	System.err.println("**** MembershipMon.getState called");
	return null;
    }

    public void setState(iBusURL channel, Serializable state)
    {
	System.err.println("**** MembershipMon.setState setState");
    }

    
    public void viewChange(View newView)
    {
	System.err.println("**** MembershipMon.viewChange for channel "
			   + newView.getChannel());

	System.err.println("**** MembershipMon.viewChange. Members(" 
			    + newView.getMembers().length + "):");
	for (int i = 0; i < newView.getMembers().length; i++) {
	    ChannelMember m = newView.getMembers()[i];
	    System.err.println("****     " 
				+ m.getURL().getAddress()
				+(m.isListener() ? " listener " : " ")
				+(m.isTalker() ? " talker" : ""));
	}
    }
}
