package iBus.example.tcp;

import java.io.Serializable;
import iBus.*;
import iBus.exception.*;

/**
 * Client to talk to a TCP time server.
 *
 * Usage: client serverURL
 * (The serverURL is printed by the server application).
 */
public class client {
    private static void usage()
    {
	System.err.println("usage: " + Application.getAppName() + " serverURL");
	System.exit(1);
    }

    public static void main(String argv[])
    {
	Application.setAppName("client");

	if (argv.length != 1) usage();

	// create a TCP stack:
	final Stack stack = new Stack("TCP");
	iBusURL url = null;
	
	// create URL:
	try {
	    url = new iBusURL(argv[0]);
	} catch (java.net.MalformedURLException e) {
	    log_.panic("main: malformed URL: " + e);
	}

	try {
	    stack.registerTalker(url);
	} catch (iBusException e) {
	    log_.panic("registerTalker failed: " + e);
	}

	// create a request posting:
	Posting request = new Posting();
	Posting replies[] = null;

	for (;;) {
	    try {
		// invoke time server:
		replies = stack.pull(url, request);
	    } catch (iBusException e) {
		System.err.println("pull failed: " + e);
		System.exit(1);
	    }

	    // display server time:
	    System.err.println("remote time: " + replies[0].getObject(0));

	    try { Thread.currentThread().sleep(5000); }
	    catch (Exception e) {}
	}
    }
    
    // Log object:
    static Log log_ = new Log("client");
}
