package iBus.example.tcp;

import iBus.exception.*;
import iBus.*;

import java.io.Serializable;
import java.util.Date;
import java.text.DateFormat;

/**
 * A simple TCP server. Waits for pull requests and returns the current time.
 * Usage: server
 *
 */
public class server {
    public static void main(String argv[])
    {
	Application.setAppName("server");

	// create a TCP stack:
	final Stack stack = new Stack("TCP");

	// create a TCP URL with the localhost address and the next
	// available port:
	final iBusURL url = iBusURLFactory.create("TimeServer", 
						  "1.0", "/timeofday", 
						  Application.getHostAddress(),
						  0);
	
	// create a server object to dispatch pull requests:
	TimeServer tsrv = new TimeServer();
	
	try {
	    stack.subscribe(url, tsrv);
	} catch (Exception e) {
	    log_.panic("main: cannot subscribe to " + url + ": " + e);
	}

	// print the URL such that it can be passed to the client
	// on the command line:
	System.err.println("The server URL:");
	System.err.println(url);

	stack.waitTillExit();
    }
    
    // Log object:
    static Log log_ = new Log("server");
}

/**
 * This class is needed for receiving pull requests sent by the
 * TCP clients.
 */
class TimeServer implements iBus.Receiver {
    public void dispatchPush(iBusURL source, Posting p)
    {
    }

    // reply to client requests:
    public Posting dispatchPull(iBusURL channel, Posting request)
    {
	System.err.println("dispatchPull: got a pull request from "
			   + request.getSender().getHostName());
	
	// get current date and time:
	final String date 
	  = DateFormat.getDateTimeInstance(DateFormat.SHORT,
					   DateFormat.SHORT).format(new Date());

	// create reply posting:
	final Posting reply = new Posting();
	reply.setLength(1);
	reply.setObject(0, date);

	// return reply to the client:
	return reply;
    }
    
    public void error(iBusURL url, String details)
    {
	System.err.println("Error: " + details);
	System.err.flush();
	System.exit(1);
    }
}
